package org.qatools.wait;

import java.util.Arrays;
import java.util.List;
import org.qatools.util.QAToolsUtil;

/**
 * 
 * @author scottwilson
 *
 */
public abstract class WaitUtil extends QAToolsUtil {
    public static long DEFAULT_SLEEP_INTERVAL_SECONDS = 100;

    // ----------------------------------------------------------------------------------------------------------------
    // -----
    // ----------------------------------------------------------------------------------------------------------------
    

    /**
     * 
     * @param timeoutSeconds
     * @param msg
     * @param waitHelperArg
     * @param waitHelper
     * @throws Exception
     */
    public static void wait(long timeoutSeconds, String msg, Object waitHelperArg, WaitHelper waitHelper) throws Exception {
        wait(timeoutSeconds, DEFAULT_SLEEP_INTERVAL_SECONDS, msg, waitHelperArg, waitHelper);
    }
    

    /**
     * 
     * @param timeoutSeconds
     * @param msg
     * @param waitHelperArg
     * @param waitHelperArg2
     * @param waitHelper
     * @throws Exception
     */
    public static void wait(long timeoutSeconds, String msg, Object waitHelperArg, Object waitHelperArg2, WaitHelper waitHelper) throws Exception {
        wait(timeoutSeconds, DEFAULT_SLEEP_INTERVAL_SECONDS, msg, waitHelperArg, waitHelperArg2, waitHelper);
    }


    /**
     * 
     * @param timeoutSeconds
     * @param msg
     * @param waitHelperArgs
     * @param waitHelper
     * @throws Exception
     */
    public static void wait(long timeoutSeconds, String msg, List<Object> waitHelperArgs, WaitHelper waitHelper) throws Exception {
        wait(timeoutSeconds, DEFAULT_SLEEP_INTERVAL_SECONDS, msg, waitHelperArgs, waitHelper);
    }

    
    // ----------------------------------------------------------------------------------------------------------------
    // -----
    // ----------------------------------------------------------------------------------------------------------------

    

    /**
     * 
     * @param timeoutSeconds
     * @param sleepIntervalSeconds
     * @param msg
     * @param waitHelperArg
     * @param waitHelper
     * @throws Exception
     */
    public static void wait(long timeoutSeconds, long sleepIntervalSeconds, String msg, Object waitHelperArg, WaitHelper waitHelper) throws Exception {
        wait(timeoutSeconds, sleepIntervalSeconds, msg, Arrays.asList (waitHelperArg), waitHelper);
    }

    /**
     * 
     * @param timeoutSeconds
     * @param sleepIntervalSeconds
     * @param msg
     * @param waitHelperArg
     * @param waitHelperArg2
     * @param waitHelper
     * @throws Exception
     */
    public static void wait(long timeoutSeconds, long sleepIntervalSeconds, String msg, Object waitHelperArg, Object waitHelperArg2, WaitHelper waitHelper) throws Exception {
        wait(timeoutSeconds, sleepIntervalSeconds, msg, Arrays.asList (waitHelperArg, waitHelperArg2), waitHelper);
    }

    
    /**
     * 
     * @param timeoutSeconds
     * @param sleepIntervalSeconds
     * @param msg
     * @param waitHelperArgs
     * @param waitHelper
     * @throws Exception
     */
    public static void wait(long timeoutSeconds, long sleepIntervalSeconds, String msg, List<Object> waitHelperArgs, WaitHelper waitHelper) throws Exception {
        Object[] tmpWaitHelperArgs = null;
        if (waitHelperArgs != null) {
            tmpWaitHelperArgs =  waitHelperArgs.toArray();
        }          
        long start = System.currentTimeMillis();
          long elapsed = -1;
          while (elapsed < timeoutSeconds) {
              if (waitHelper.checkCondition(tmpWaitHelperArgs)) {
                  System.out.println("Condition is met - " + msg);
                  return;
              }
              try {
                  Thread.sleep(sleepIntervalSeconds);
              } catch (InterruptedException ignored) {
              }
              elapsed = (System.currentTimeMillis() - start) / 1000;
          }
          System.out.println("Timeout - " + msg);        
          throw new Exception("Timeout - " + msg);
    }
    

    
    
}